@@ -55,7 +55,7 @@ class Model:
 
     To make it work, you can provide:
 
-    :Override ``serie_name()`` if needed:
+    :Override ``_serie_name()`` if needed:
         You return the serie name. This value will be used to prefix keys in DB.
         Default is the class' qualified name (``__qualname__``).
 
@@ -99,13 +99,44 @@ class Model:
     Python ``del``, instead, set its value to ``None``.
     '''
 
-    '''This is reserved key.'''
-    _ID = '_id'
-    '''This is reserved key.'''
-    _COUNT = '_count'
+    ### SETTINGS
+
     _AUTO_ID = True
 
-    # SOME CONSTANTS
+    '''Default set size that Redis supports. You can change this for your needs.
+    However this value does *not* affect indexes (which use sorted set).'''
+    _SET_SIZE = 2**32 - 1
+
+    '''Add all fields that you want them to be indexed to this list. If you do,
+    you might consider overriding function ``_score()`` to support your values.
+    '''
+    _INDEXES = []
+
+    '''This is *not* maximum length of strings that you can store. Redis allows
+    up to 512 MiB.
+
+        This is the maximum length of strings that can be *calculated* their
+        score. If a string length is larger than this value, then only the first
+        x characters are used to calculate the strings' score. Of course ``x``
+        is the value of this flag.
+
+        Be careful if you want to change this. All old scores will be invalid
+        with new value.
+    '''
+    _MAX_STR_LEN = 99
+
+    '''Convenient value for maximum string's score.'''
+    _MAX_STR_SCORE = sys.float_info.max
+
+    '''Sets to ``True`` to compares strings with case. Default is ``False``.'''
+    _STR_CASE_SENSITIVE = False
+
+    ### CONSTANTS
+
+    _S_ID = '_id'
+    _S_IDS = '_ids'
+    _S_INDEXES = '_indexes'
+
     _S_YEAR = '_year'
     _S_MONTH = '_month'
     _S_DAY = '_day'
@@ -116,20 +147,20 @@ class Model:
     _S_UTC_OFFSET = '_utc_offset'
 
     @classmethod
-    def serie_name(cls):
+    def _serie_name(cls):
         '''Returns the serie name of this object in DB.
         '''
         return cls.__qualname__
 
     @classmethod
-    def uid(cls, inc=True):
+    def _uid(cls, inc=True):
         '''Returns the current unique ID of the model. If ``inc`` is ``True``,
         increases the ID by ``1`` and returns new value.
         '''
 
-        _ = SEP.join([cls.serie_name(), cls._ID])
+        _ = SEP.join([cls._serie_name(), cls._S_ID])
         return R.incr(_) if inc else int(R.get(_).decode())
-        #.uid()
+        #._uid()
 
     @classmethod
     def _redis2python(cls, name, key, obj):
@@ -193,16 +224,18 @@ class Model:
         #._redis2python()
 
     @classmethod
-    def query(cls, func, *names, id_gt=0, id_lt=0, limit=0):
+    def query(cls, func=None, *names, id_gt=None, id_lt=None, limit=None,
+              order=None):
         '''Queries.
 
         Parameters:
 
         :func:
             The callback function which accepts one argument -- which is an
-            arbitrary object crarrying all fields provided in ``names``. You
-            return ``True`` to accept the result; return ``None`` or ``False``
-            otherwise.
+            arbitrary object specified by ``names``. You return ``True`` to
+            accept the result; return ``None`` or ``False`` otherwise.
+
+            If not provided, all items will be returned.
 
         :*names:
             List of variable names of current instance of this model that you
@@ -210,76 +243,213 @@ class Model:
             arbitrary object, that object will be passed back to ``func`` for
             you to process.
 
+            If not provided, the *real* record will be passed to ``func``.
+
         :id_gt:
-            If set, and the current instance of this model enables auto ID, then
-            *try* to only looks for items having ID greater than the given
+            If set then only looks for items having ID greater than the given
             value.
 
         :id_lt:
-            If set, and the current instance of this model enables auto ID, then
-            *try* to only looks for items having ID lower than the given value.
+            If set then only looks for items having ID lower than the given
+            value.
 
         :limit:
-            To limits the results. ``0`` is unlimited.
+            To limits the results. ``None`` is unlimited.
+
+        :order:
+            This is a tuple of 3, 4 or 6. It can be represented by names like:
+            ``order=(name,min,max,offset,count,ascending)``. Where:
+            * ``name`` is the name of field you want to order.
+            * ``min`` is the minimum score.
+            * ``max`` is the maximum score.
+            * ``ascending`` (optional), default is ``True``, which will order
+              the result from low to high scores. Set to ``False`` or ``None``
+              to use descending order.
+            * ``offset`` (optional), is the offset. If set, ``count`` must be
+              set too.
+            * ``count`` (optional), is the count.
 
         Returns:
-            A list of "accepted objects" which are specified by ``func``. If
-            current instance of the model support auto ID, the real items will
-            be returned. Otherwise the empty objects which carry variables you
-            required will be returned.
+            A list of "accepted objects" which are specified by ``func`` and
+            ``names``.
 
-        Note that if you use both ``id_gt`` and ``id_lt``, they should not be
-        conflicted. For example this is wrong: ``id_gt=99` and ``id_lt=9``. In
-        such case, an empty list is returned.
+        Notes:
+
+            If you use both ``id_gt`` and ``id_lt``, they should not be
+            conflicted. For example this is wrong: ``id_gt=99` and ``id_lt=9``.
+            In such case, an empty list is returned.
+
+            If current instance does not support auto ID, an empty list is
+            returned.
         '''
 
+        if not cls._AUTO_ID or (id_gt != None and id_lt != None and id_gt >= id_lt):
+            return []
+
         # BUILD THE KEY PREFIX
 
-        prefix = cls.serie_name() + SEP
-
-        if cls._AUTO_ID:
-            if id_gt and id_lt:
-                if id_gt >= id_lt: return []
-                prefix += '?' * len(str(id_gt)) + '*'
-            elif id_gt:
-                prefix += '?' * len(str(id_gt)) + '*'
-            elif id_lt:
-                prefix += '?' * len(str(id_gt))
-            else:
-                prefix += '*'
-            prefix += SEP
+        prefix = cls._serie_name()
 
         # QUERY
 
         result = []
-        for name in names:
-            for key in R.keys(prefix + name):
-                key = key.decode()
+        id_set = 0
+        while 1:
+            if order:
+                key_ids = SEP.join([cls._serie_name(), cls._S_INDEXES, order[0]])
+                if not R.zcard(key_ids): return result
 
-                obj = Empty()
-                cls._redis2python(name, key, obj)
+                offset = None if len(order) <= 4 else order[4]
+                count = None if len(order) <= 5 else order[5]
 
-                if func(obj):
-                    if cls._AUTO_ID:
-                        _ = key[:-len(name)-len(SEP)]
-                        _ = int(re.search(r'[0-9]+$', _).group())
-                        result.append(cls(_id=_))
+                if len(order) <= 3 or order[3]:
+                    keys = R.zrangebyscore(key_ids, order[1], order[2], offset,
+                                           count)
+                else:
+                    keys = R.zrevrangebyscore(key_ids, order[2], order[1],
+                                              offset, count)
+            else:
+                key_ids = SEP.join([cls._serie_name(), cls._S_IDS, str(id_set)])
+                id_set += 1
+                if not R.scard(key_ids): return result
+                keys = R.smembers(key_ids)
+
+            for key in keys:
+                key = int(key.decode())
+                if id_gt != None and key <= id_gt: continue
+                if id_lt != None and key >= id_lt: continue
+
+                if names:
+                    obj = Empty()
+                    _ = SEP.join([prefix, str(key), ''])
+                    for name in names:
+                        cls._redis2python(name, _ + name, obj)
+
+                    if func:
+                        if func(obj):
+                            result.append(cls(_id=key))
                     else:
                         result.append(obj)
                     if limit and len(result) >= limit:
                         return result
+                else:
+                    obj = cls(_id=key)
+                    if func:
+                        if func(obj): result.append(obj)
+                    else: result.append(obj)
+                #.for
+
+            # ``while`` loop is for ID set only. For indexes we just need this
+            # loop once.
+            if order: return result
+            #.while
 
         return result
         #.query()
 
+    @classmethod
+    def _add_id(cls, _id):
+        '''Adds an ID to ID set. This does nothing if ``_AUTO_ID`` is ``False``,
+        or ``_id < 0``.
+        '''
+
+        if _id < 0: return
+
+        i = 0
+        while 1:
+            key = SEP.join([cls._serie_name(), cls._S_IDS, str(i)])
+            if R.scard(key) < cls._SET_SIZE:
+                R.sadd(key, _id)
+                return
+            i += 1
+            #.while
+        #._add_id()
+
+    @classmethod
+    def _remove_id(cls, _id):
+        '''Remove an ID from ID set. This does nothing if ``_AUTO_ID`` is
+        ``False``, or ``_id < 0``.'''
+
+        if _id < 0: return
+
+        i = 0
+        while 1:
+            key = SEP.join([cls._serie_name(), cls._S_IDS, str(i)])
+            if R.sismember(key, _id):
+                R.srem(key, _id)
+                return
+            i += 1
+            #.while
+        #._remove_id()
+
     @classmethod
     def count(cls):
         '''Counts total item of current model.
         '''
-        _ = R.get(SEP.join([cls.serie_name(), cls._COUNT]))
-        return int(_.decode()) if _ else 0
+
+        result = i = 0
+        while 1:
+            size = R.scard(SEP.join([cls._serie_name(), cls._S_IDS, str(i)]))
+            if size > 0:
+                result += size
+                i += 1
+            else: break
+
+        return result
         #.count()
 
+    @classmethod
+    def _score(cls, obj):
+        '''Override this function to "score" your values. By default, this
+        function supports some numeric objects such as ``int``, ``float``,
+        ``complex``, ``bool``; ``datetime``, ``date`` and ``time`` in module
+        ``datetime``.
+
+        For ``str``s, this function turns them to lower case, then calculates
+        their value by summing up all characters.
+
+        For groups (``list``, ``set``, ``dict``, ``bytes``...), this function
+        calculates their value by their size.
+
+        For module ``datetime``'s types: ``datetime`` and ``date``'s values are
+        total seconds from ``datetime.min``. ``time``'s value is total seconds
+        it holds.
+
+        This function returns ``0`` for unsupported types.
+        '''
+
+        _type = type(obj)
+
+        if _type in [dict, set, list, bytes]:
+            return len(obj)
+        elif _type == datetime.datetime:
+            return (obj - datetime.min).total_seconds()
+        elif _type == datetime.date:
+            return (obj - date.min).total_seconds()
+        elif _type == datetime.time:
+            result = obj.hour * 60 * 60 + obj.minute * 60 + obj.second + \
+                     obj.microsecond / 1000
+            if obj.tzinfo:
+                result += obj.tzinfo.utcoffset(None).total_seconds()
+            return result
+        elif _type in [int, float]:
+            return obj
+        elif _type == bool:
+            return int(obj)
+        elif _type == str:
+            # We assume all strings take the same _MAX_STR_LEN length. So we
+            # append all strings with "virtual" characters to fit _MAX_STR_LEN.
+            # Every "virtual" character has a value of 0.
+            if not cls._STR_CASE_SENSITIVE:
+                obj = obj.lower()
+            result = 0
+            for i in range(0, len(obj)):
+                result += ord(obj[i]) * 10**(cls._MAX_STR_LEN - i)
+            return result
+
+        return 0
+        #._score()
+
     def __init__(self, __strict=True, **args):
         '''Creates new instance. ``args`` is a list of key-value pairs.
 
@@ -294,8 +464,8 @@ class Model:
         when you call ``put()``.
         '''
 
-        if len(args) == 1 and Model._ID in args:
-            self._id = args[Model._ID]
+        if len(args) == 1 and __class__._S_ID in args:
+            self._id = args[__class__._S_ID]
 
             # Try to get the item in DB
             name_count = missing_names = 0
@@ -305,7 +475,7 @@ class Model:
                 attr = getattr(self.__class__, name)
                 if callable(attr) and not isinstance(attr, type): return
 
-                key = SEP.join([self.__class__.serie_name(), str(self._id), name])
+                key = SEP.join([self.__class__._serie_name(), str(self._id), name])
                 name_count += 1
                 if R.exists(key):
                     self.__class__._redis2python(name, key, self)
@@ -314,7 +484,7 @@ class Model:
                 #.for
 
             if __strict and name_count > 0 and missing_names == name_count:
-                raise KeyError
+                raise KeyError('{:,}'.format(self._id))
 
             return
             #.if
@@ -327,10 +497,14 @@ class Model:
         '''Put this item into DB.
         '''
 
-        key = self.__class__.serie_name() + SEP
+        pipe = R.pipeline()
+
+        key = self.__class__._serie_name() + SEP
         if self.__class__._AUTO_ID:
-            if self._id < 0: self._id = self.uid()
+            if self._id < 0: self._id = self.__class__._uid()
             key += '{}{}'.format(self._id, SEP)
+            key_indexes = SEP.join([self.__class__._serie_name(),
+                                    __class__._S_INDEXES, ''])
 
         key_count = 0
         change_count = 0
@@ -341,79 +515,104 @@ class Model:
 
             key_count += 1
             value = allvars[name]
+
+            if self._id >= 0 and hasattr(self.__class__, name) and \
+                getattr(self.__class__, name) in self.__class__._INDEXES:
+                key_index = key_indexes + name
+                if value != None:
+                    pipe.zadd(key_index, self.__class__._score(value), self._id)
+                else:
+                    pipe.zrem(key_index, self._id)
+
             name = key + name
 
             _type = type(value)
             if _type == dict:
-                R.hmset(name, value)
+                pipe.hmset(name, value)
                 change_count += 1
             elif _type == set:
-                R.sadd(name, *value)
+                pipe.sadd(name, *value)
                 change_count += 1
             elif _type == list:
-                R.delete(name)
-                R.rpush(name, *value)
+                pipe.delete(name)
+                pipe.rpush(name, *value)
                 change_count += 1
             elif _type == datetime.datetime:
-                R.hmset(name, {
-                        __class__._S_YEAR: value.year,
-                        __class__._S_MONTH: value.month,
-                        __class__._S_DAY: value.day,
-                        __class__._S_HOUR: value.hour,
-                        __class__._S_MINUTE: value.minute,
-                        __class__._S_SECOND: value.second,
-                        __class__._S_MICROSECOND: value.microsecond })
+                pipe.hmset(name, {
+                           __class__._S_YEAR: value.year,
+                           __class__._S_MONTH: value.month,
+                           __class__._S_DAY: value.day,
+                           __class__._S_HOUR: value.hour,
+                           __class__._S_MINUTE: value.minute,
+                           __class__._S_SECOND: value.second,
+                           __class__._S_MICROSECOND: value.microsecond })
                 if value.tzinfo:
-                    R.hset(name, __class__._S_UTC_OFFSET,
-                           int(value.tzinfo.utcoffset(None).total_seconds()))
+                    pipe.hset(name, __class__._S_UTC_OFFSET,
+                              int(value.tzinfo.utcoffset(None).total_seconds()))
                 change_count += 1
             elif _type == datetime.date:
-                R.hmset(name, {
-                        __class__._S_YEAR: value.year,
-                        __class__._S_MONTH: value.month,
-                        __class__._S_DAY: value.day })
+                pipe.hmset(name, {
+                           __class__._S_YEAR: value.year,
+                           __class__._S_MONTH: value.month,
+                           __class__._S_DAY: value.day })
                 change_count += 1
             elif _type == datetime.time:
-                R.hmset(name, {
-                        __class__._S_HOUR: value.hour,
-                        __class__._S_MINUTE: value.minute,
-                        __class__._S_SECOND: value.second,
-                        __class__._S_MICROSECOND: value.microsecond })
+                pipe.hmset(name, {
+                           __class__._S_HOUR: value.hour,
+                           __class__._S_MINUTE: value.minute,
+                           __class__._S_SECOND: value.second,
+                           __class__._S_MICROSECOND: value.microsecond })
                 if value.tzinfo:
-                    R.hset(name, __class__._S_UTC_OFFSET,
-                           int(value.tzinfo.utcoffset(None).total_seconds()))
+                    pipe.hset(name, __class__._S_UTC_OFFSET,
+                              int(value.tzinfo.utcoffset(None).total_seconds()))
                 change_count += 1
             elif value != None:
-                R.set(name, value)
+                pipe.set(name, value)
                 change_count += 1
             else:
-                R.delete(name)
+                pipe.delete(name)
                 change_count -= 1
             #.for
 
-        _ = SEP.join([self.__class__.serie_name(), __class__._COUNT])
+        # ADD OR REMOVE _id to ID SET
+
         if key_count + change_count == 0:
-            R.decr(_)
+            self.__class__._remove_id(self._id)
         else:
-            R.incr(_)
+            self.__class__._add_id(self._id)
+
+        pipe.execute()
         #.put()
 
     def delete(self):
-        '''Deletes item by its ID. Returns ``True`` if OK, ``None`` otherwise.
+        '''Deletes this item. Returns ``True`` if OK, ``None`` otherwise.
         '''
 
-        if self._id < 0: return
-
         keys = []
+        keys_indexes = None
+        if self.__class__._AUTO_ID and self._id >= 0 and self.__class__._INDEXES:
+            keys_indexes = []
+
         for name in vars(self.__class__):
             if name.startswith('_'): continue
 
             attr = getattr(self.__class__, name)
             if callable(attr) and not isinstance(attr, type): continue
 
-            keys.append(SEP.join([self.__class__.serie_name(), str(self._id), name]))
+            if self._id >= 0:
+                _ = SEP.join([self.__class__._serie_name(), str(self._id), name])
+                if keys_indexes != None and attr in self.__class__._INDEXES:
+                    keys_indexes.append(name)
+            else:
+                _ = SEP.join([self.__class__._serie_name(), name])
+            keys.append(_)
             #.for
 
-        if keys: R.delete(*keys)
-        R.decr(SEP.join([self.__class__.serie_name(), __class__._COUNT]))
+        if keys:
+            R.delete(*keys)
+            self.__class__._remove_id(self._id)
+        if keys_indexes:
+            _ = SEP.join([self.__class__._serie_name(), __class__._S_INDEXES, ''])
+            for name in keys_indexes:
+                R.zrem(_ + name, self._id)
         #.delete()
